import 'dart:async';
import 'dart:convert' as convert;
import 'package:flutter/services.dart';
import 'package:witsystem_device_plugin/MethodName.dart';
import 'package:witsystem_device_plugin/WitsystemSDK.dart';

/// 添加设备

class AddDevice extends WisystemSDk {
  final Stream<Map<String, dynamic>> _restoreStateEvents =
      const EventChannel(ChannelName.multipleAddDevice)
          .receiveBroadcastStream()
          //   .handleError((errorJson) {
          // var e = errorJson as PlatformException;
          // throw  convert.json.decode(e.details);
          //    })
          .map((event) => convert.json.decode(event.toString()));

  factory AddDevice() => _getInstance();

  static AddDevice get instance => _getInstance();
  static AddDevice? _instance;

  AddDevice._internal();

  static AddDevice _getInstance() {
    _instance ??= AddDevice._internal();
    return _instance!;
  }

  StreamController<Map<String, dynamic>>? _scanStreamController;

  ///扫描可以添加的设备
  Stream<Map<String, dynamic>> scanAddBleDevice() {
    //判断没有关闭将继续使用
    if (_scanStreamController != null && !_scanStreamController!.isClosed) {
      return _scanStreamController!.stream;
    }
    _scanStreamController = StreamController.broadcast(onListen: () {
      //启动监听启动扫描
      methodChannel
          .invokeMethod(MethodName.scanAddBleDevice)
          .catchError((err) => {})
          .then((value) => {});
    }, onCancel: () {
      methodChannel.invokeMethod(MethodName.stopAddBleDevice); //取消监听停止扫描
    });
    _scanStreamController
        ?.addStream(_restoreStateEvents, cancelOnError: true)
        .then((value) {
      _scanStreamController?.close();
    });
    return _scanStreamController!.stream;
  }

  ///停止扫描添加设备
  stopAddBleDevice(StreamSubscription? listen) async {
    if (listen != null) listen.cancel(); //取消监听自动停止扫描
  }

  ///添加
  ///waitSettingsState 等待设备进入设置状态
  addBleDevice(String deviceId, String deviceName,
      {Function(String deviceId)? waitSettingsState}) async {
    if (waitSettingsState != null) {
      methodChannel.setMethodCallHandler((call) {
        if (call.method == "waitSettingsState") {
          waitSettingsState(call.arguments);
          methodChannel.setMethodCallHandler(null);
        }
        return Future.value("SUCCESS");
      });
    }
    final map = {"deviceId": deviceId, "deviceName": deviceName};
    try {
      return await methodChannel.invokeMethod(MethodName.addBleDevice, map);
    } catch (e) {
      methodChannel.setMethodCallHandler(null);
      rethrow;
    }
  }
}
